# This example code illustrates the coding strategy in the complete 
# code appendix for the main analysis of recurrence in the largest 
# network in Figure 1. The complete code does all these steps 
# automatically for all networks (i.e., connected subgraphs) and 
# for all outcomes, handling exceptions as needed. 
 
# Read necessary packages. 
library(metafor)

# Read the data in to a dataframe 
data <- read.csv("_d_RecurBCC_cleandata_subgraph1.csv")
data <- as.data.frame(data)

# Create a copy of the "treatment" variable, and pick a reference
# treatment. This is unimportant in terms of fitting the main 
# analysis. However, code such as this will be used to change 
# the reference treatment to easily get specific results. 
data[, "trt"] <-  data[, "treatment"]
reference_treatment_name <- 
    sort(levels(data[, "trt"])[as.numeric(data.ab[, "trt"])])[1]
data <- 
    within(data, trt<-relevel(trt, ref=reference_treatment_name))

# Estimate the empirical log-odds as logit(responders/sampleSize).
# The escalc() function of metafor does this handling edge cases
data[, c("yi", "vi")] <- 
    escalc(measure="PLO", xi=data[, "responders"], 
           ni=data[, "sampleSize"]) 

# Run the analysis in metafor using restricted maximum likelihood. 
# Because this analysis has at least 3 treatments and at least 
# 4 RCTs in a comparison, we fit a random effects model. 
# Otherwise, we would fit a fixed effects model.
# In the complete code, we count the treatments and the trials per
# comparison by manipulating a mathematical graph object, and 
# fork the analysis accordingly (not shown in this example).  
# This command fits the normal-normal models described in the 
# theory section of the technical appendix. 
results <- rma.mv(yi=yi, vi=vi, mods= as.formula(" ~ 1 + trt"), 
                      data=data, 
                      random= as.formula(" ~ 1 | study / trt"), 
                      method="REML")


# League tables: 

# Because this analysis is modeling the empirical log-odds of the 
# event in each arm, we can estimate not only the mean effect 
# across studies (i.e.,  the mean difference in the log-odds)
# but also the mean frequency of the event with each treatment. 
# We can get these results for the above analysis as "predictions" 
# of the mean proportion in each treatment. 
# Note that in hierarchical models many different predictions can 
# be obtained. This code with metafor gets us the predictions we 
# need. 

  ## To wit, first, set up an elementary design matrix to obtain 
  ## predictions; This is an identity matrix I of the appropriate rank,
  ## stacked on a row-vector of 0's
  ## 
  ##                          | 1, 0, ..., 0, 0 |
  ##              | I |       | 0, 1, ..., 0, 0 |
  ##  contrasts = +---+ =     | ...             |
  ##              | 0 |       | 0, 0, ..., 0, 1 |
  ##                          +-----------------+
  ##                          | 0, 0, ..., 0, 0 |
  ##
  ## 
  ## The last row of this matrix will get us the mean log-odds for the
  ## event in the reference treatment, which corresponds to the intercept
  ## of the model in rows 39-42.The t-th row of the identity matrix obtains 
  ## the mean log-odds for the t-th non-reference treatment.  
  ## This design matrix works because we specified "intercept=TRUE" in line 77. 
  n_treatments <- length(results$b)
  contrasts <- diag(n_treatments-1)
  contrasts <- rbind( rep(0, n_treatments-1), contrasts )
  logit_means <- predict(results, newmods=contrasts, intercept=TRUE )

  ## Then, we have to backcalculate the mean proportions from the mean odds 
  ## Some of this code is admittedly inefficient manipulation of objects in R. 
  ## Coerce the list.rms object to a dataframe by taking the output of print()!
  logit_means <- as.data.frame(print.list.rma(logit_means))
  ## Proportion mean and confidence interval with pointwise transformation. 
  pred <- exp(logit_means)/(1+exp(logit_means))   

# Relative effects tables:  

  ## The coefficients in the "results" object in lines 39-42
  ## correspond to a column of the relative effects table, namely the 
  ## column that corresponds to the reference treatment chosen in 

  ## There are several ways to obtain the full relative effects table. 
  ## an efficient one would be to create an appropriate elementary 
  ## design matrix with each row corresponding to each relative 
  ## effect and run a command analogous to line 77. 

  ## However, we obtained the relative effects table the following way:
  ## 1. by swithching the reference treatment in the "trt" factor 
  ##    using code analogous to that in lines 21-22
  ## 2. by re-running the model in 39-42 for the new reference 
  ##    treatment, to obtain another column of the relative effects 
  ##    table 
  ##
  ## 1 and 2 are accomplished in a dedicated function in the R code. 
  ##
  ## While this roundabout is not the optimal approach 
  ## (is not numerically stable in edge cases) we believe that it 
  ## did not introduce problems in the current analysis. 


# "Split-node" analyses: 

  ## The name is unfortunate, because it refers to the directed acyclic 
  ## graphs underlying the MCMC algorithm in Bayesian models, which 
  ## correspond to edges in network figures such as Figures 1-4 in the 
  ## paper. Here is what this analysis does: 
  ##
  ## Consider the small network 
  ##    A --- B
  ##     \  /  \  .
  ##      C ---D
  ## 
  ## Consistency assumes that the indirect effect for the comparison of
  ## say, treatment A vs treatment C (i.e., the effect we estimate 
  ## without considering any A vs C trials) is the same as the direct 
  ## effect from the A vs C trials. 
  ## In the augmented network below, in the A vs C trials we 
  ## renamed C as C* :
  ## 
  ##    A --- B
  ##   /    /  \  .
  ##  C*  C ---D
  ##
  ## We can prove (but do not show the proof) that running a network 
  ## meta-analysis in this augmented network gets us
  ##   1. an estimate of the direct effect of treatment A vs treatment C  
  ##      in the form of the effect (A vs C*)
  ##   2. an estimate of the indirect effect of treatment A vs treatment C
  ##      based on all other data in the network in the form of the 
  ##      effect (A vs C).
  ##
  ## We can then get the difference of direct and indirect effects 
  ## from the relative effects table for the augmented network.
  ##
  ## The catch is that one has to do several such analyses, one for every 
  ## informationally independent cycle in the network (many authors 
  ## of network meta-analyses refer to the cycles in a graph as "loops"). 
  ## The algorithm  to do this involves some maths on graphs. 
  ## We do not describe the intuition behind these maths.  




